/*****************************************************
Date    : 2013-12-20
Chip type           : ATmega16L
Clock frequency     : 11.059200 MHz
LUKE   xiechenglin  13760249474
QQ:35284316
E-mail:13760249474@139.com
*****************************************************/
#include "main.h"



interrupt [EXT_INT0] void ext_int0_isr(void)
{
    GIFR=0x40;
    SiRevIntFlag=1;
}

interrupt [USART_RXC] void usart_rx_isr(void)
{
    if(SiSendFlag==0)
    {
	    RevUartBuf[revCnt++]=UDR;
	    if( (RevUartBuf[revCnt-2]==0x0d)&&(RevUartBuf[revCnt-1]==0x0a) )
        {
            SiSendFlag=1;
        }
        if(revCnt>126)revCnt=0;
    }
}



void Io_Init(void)
{
    // Input/Output Ports initialization
    // Port A initialization
    // Func7=In Func6=In Func5=In Func4=In Func3=Out Func2=Out Func1=Out Func0=Out 
    // State7=T State6=T State5=T State4=P State3=0 State2=0 State1=1 State0=1 
    PORTA=0x13;
    DDRA=0x0F;

    // Port B initialization
    // Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=Out 
    // State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=0 
    PORTB=0x00;
    DDRB=0x01;

    // Port C initialization
    // Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
    // State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
    PORTC=0x00;
    DDRC=0x00;

    // Port D initialization
    // Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
    // State7=P State6=T State5=T State4=T State3=T State2=P State1=T State0=T 
    PORTD=0xff;
    DDRD=0x00;
}

void Uart_Init(void)
{
    // USART initialization
    // Communication Parameters: 8 Data, 1 Stop, No Parity
    // USART Receiver: On
    // USART Transmitter: On
    // USART Mode: Asynchronous
    // USART Baud Rate: 9600 (Double Speed Mode)
    UCSRA=0x02;
    UCSRB=0x98;
    UCSRC=0x86;
    UBRRH=0x00;
    UBRRL=0x8F;
}


void Ext_Inter_Init(void)
{
    // External Interrupt(s) initialization
    // INT0: On
    // INT0 Mode: Falling Edge
    // INT1: Off
    // INT2: Off
    GICR|=0x40;
    MCUCR=0x02;
    MCUCSR=0x00;
    GIFR=0x40;
}

void UartSendByte(unsigned char ch)
{
	while(!(UCSRA&(1<<5)));
	UDR=ch;
}

void UartSendStr(unsigned char *str)
{
	while(*str)
	{
		UartSendByte(*str++);
	}
}

void UartSendBuf(unsigned char *buf,unsigned long bufLength)
{
	unsigned long i;
	for(i=0;i<bufLength;i++)
	{
		UartSendByte(buf[i]);
	}
}


void Key_Scan(void)
{
    unsigned char buf[8]={0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08};
     if(KEY==0)
     {
        delay_ms(10);
        if(KEY==0)
        {
            while(KEY==0);
            LED=~LED;
            Si4432_Send_Data(buf,sizeof(buf));
        }
     }
}




void Si4432_Write_Register(unsigned char regAddr,unsigned char regVol)
{
    unsigned char i;
    regAddr|=0x80;	
   
    SI_NSEL=0;                              
    delay_us(2);
    for(i=0;i<8;i++)
    {
        if(regAddr&0x80)
        {
            SI_SDO=1;
        }
        else
        {
            SI_SDO=0;
        }
        SI_NSCK=1;
        delay_us(2);
        SI_NSCK=0; 
        delay_us(1);           
        regAddr=regAddr<<1;
    }
    for(i=0;i<8;i++)
    {
        if(regVol&0x80)
        {
            SI_SDO=1;
        }
        else
        {
            SI_SDO=0;
        }
        SI_NSCK=1;
        delay_us(2);
        SI_NSCK=0; 
        delay_us(1);           
        regVol=regVol<<1;
    }
	delay_us(2);
	SI_NSEL=1;    
}




unsigned char Si4432_Read_Register(unsigned char regAddr)
{
    unsigned char i;
	unsigned char Result=0;
	regAddr&=0x7f;
    
    SI_NSEL=0; 	
	delay_us(2);
    for(i=0;i<8;i++)
    {
        if(regAddr&0x80)
        {
            SI_SDO=1;
        }
        else
        {
            SI_SDO=0;
        }
        SI_NSCK=1;
        delay_us(2);
        SI_NSCK=0; 
        delay_us(1);           
        regAddr=regAddr<<1;
    }	
    for(i=0;i<8;i++)
    {                   
        Result=Result<<1;
  	 	SI_NSCK=1;
  	 	delay_us(1);         
        if(SI_SDI)
        {
            Result|=0x01;
        }
        SI_NSCK=0;
  	 	delay_us(1);        
    }
  	delay_us(2);
	SI_NSEL = 1;                            
    return Result;
}


void Si4432_Send_Data(unsigned char *buf,unsigned char len)
{
    unsigned char i=0;
    
    //ý
    Si4432_Write_Register(0x07,0x01);																

    //ݰ	
    Si4432_Write_Register(0x3E,len);	
    		
    //ݰ͵FIFO
    for(i=0;i<len;i++)
    {
        Si4432_Write_Register(0x7F,buf[i]);  
    }

    //رжϣʹжֻ.ָʾ
    Si4432_Write_Register(0x05,0x04);	
    Si4432_Write_Register(0x06,0x00);	
            
            
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);	
    Si4432_Read_Register(0x04);	

    //ʹ  ߵγݰԶ͡
    Si4432_Write_Register(0x07,0x09);

    //ȴжϷݰ
    while(SI_NIRQ==1);
    
    
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);
    Si4432_Read_Register(0x04);

			
    //Ϊģʽ
    //ʹж 
    // a) ӽӵݰ
    // b) յݰCRC
    Si4432_Write_Register(0x05,0x03); 
    Si4432_Write_Register(0x06,0x00); 
    
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);	
    Si4432_Read_Register(0x04);	

    //ʹܽ
    Si4432_Write_Register(0x07,0x05);	   	
}


unsigned char Si4432_Rev_Data(unsigned char *revBuf)
{
    unsigned char i=0;
    unsigned char len=0;
    unsigned char ItStatus1=0;
    unsigned char ItStatus2=0;
    unsigned long timerOut=0x3fff;
    while(timerOut!=0)
    {
        timerOut--;
        if(SI_NIRQ==0)  
        {
            //ý
            Si4432_Write_Register(0x07,0x01);
            
            //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
            ItStatus1=Si4432_Read_Register(0x03);	
            ItStatus2=Si4432_Read_Register(0x04);	
            
            //CRCжϷ
            if((ItStatus1&0x01)== 0x01)
            {
                //RX FIFO
                Si4432_Write_Register(0x08,0x02);	
                Si4432_Write_Register(0x08,0x00);	
            }
            
			//յݰж		
            if((ItStatus1&0x02)==0x02)
            {
                //ȡյЧغɵĳ
                len=Si4432_Read_Register(0x4B);	//read the Received Packet Length register
                
                //ȡreeivedЧغɴRX FIFO
                for(i=0;i<len;i++)
                {
                    revBuf[i]=Si4432_Read_Register(0x7F);	//read the FIFO Access register
                }
                UartSendBuf(revBuf,len);
            }
            
            //RX FIFO
            Si4432_Write_Register(0x08,0x02);	
            Si4432_Write_Register(0x08,0x00);	
                
            //ٴý
            Si4432_Write_Register(0x07,0x05);	
        }  
    }
    return len;
}


unsigned char Si4432_Get_Data(unsigned char *revBuf)
{
    unsigned char i=0;
    unsigned char len=0;
    unsigned char ItStatus1=0;
    unsigned char ItStatus2=0;


    //ý
    Si4432_Write_Register(0x07,0x01);
            
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    ItStatus1=Si4432_Read_Register(0x03);	
    ItStatus2=Si4432_Read_Register(0x04);	
            
    //CRCжϷ
    if((ItStatus1&0x01)== 0x01)
    {
        //RX FIFO
        Si4432_Write_Register(0x08,0x02);	
        Si4432_Write_Register(0x08,0x00);	
    }
            
    //յݰж		
    if((ItStatus1&0x02)==0x02)
    {
        //ȡյЧغɵĳ
        len=Si4432_Read_Register(0x4B);	//read the Received Packet Length register
                
        //ȡreeivedЧغɴRX FIFO
        for(i=0;i<len;i++)
        {
            revBuf[i]=Si4432_Read_Register(0x7F);	//read the FIFO Access register
        }
        UartSendBuf(revBuf,len);
    }
            
    //RX FIFO
    Si4432_Write_Register(0x08,0x02);	
    Si4432_Write_Register(0x08,0x00);	
                
    //ٴý
    Si4432_Write_Register(0x07,0x05);	

    return len;
}

void Si4432_Init(void)
{
    //ͨPWRDNŴSI4432ڲԴ
    SI_SND=0;
    
    
    //ٵȴ15beforyκγʼSPI͵SI4432
    //ȴϵ縴λ
    delay_ms(20);
       
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);
    Si4432_Read_Register(0x04);
    
    //λоƬ   
    Si4432_Write_Register(0x07,0x80);    
    
    //ȴPORжϣNIRQΪߣ
    while(SI_NIRQ==1);
    
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);
    Si4432_Read_Register(0x04);                                                 //read the Interrupt Status2 register
     
    //ȴоƬжϣNIRQΪߵƽ
    while(SI_NIRQ==1);
    
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);
    Si4432_Read_Register(0x04);  
                        
    //
    //Ƶ 915 MHz
    //Si4432_Write_Register(0x75,0x75);														  
	//Si4432_Write_Register(0x76,0xBB);															
	//Si4432_Write_Register(0x77,0x80); 
 	//д0x80زFrequency0Ĵ
	Si4432_Write_Register(0x75,0x53);															
	Si4432_Write_Register(0x76,0x64);															
	Si4432_Write_Register(0x77,0x00); 
	
	//TX (9.6kbps)
	//Si4432_Write_Register(0x6E,0x4E);															
	//Si4432_Write_Register(0x6F,0xA5);															
	//Si4432_Write_Register(0x70,0x2C);														
    
	//TX (1.2kbps)
	Si4432_Write_Register(0x2a,0x14);	
	Si4432_Write_Register(0x6E,0x09);														
	Si4432_Write_Register(0x6F,0xd5);															
	Si4432_Write_Register(0x70,0x2C);	
	
	//TXƫĴ (+-45kHz)
	//Si4432_Write_Register(0x72,0x48);	//(9.6kbps)													
    Si4432_Write_Register(0x72,0x38);	//(1.2kbps)	

	/*EXELõƽ: 9.6 kbps, deviation: 45 kHz, channel filter BW: 102.2 kHz*/
	Si4432_Write_Register(0x1C,0x1b);		//write 0x1E to the IF Filter Bandwidth register		
	Si4432_Write_Register(0x20,0x83);		//write 0xD0 to the Clock Recovery Oversampling Ratio register		
	Si4432_Write_Register(0x21,0xc0);		//write 0x00 to the Clock Recovery Offset 2 register		
	Si4432_Write_Register(0x22,0x13);		//write 0x9D to the Clock Recovery Offset 1 register		
	Si4432_Write_Register(0x23,0xa9);		//write 0x49 to the Clock Recovery Offset 0 register		
	Si4432_Write_Register(0x24,0x00);		//write 0x00 to the Clock Recovery Timing Loop Gain 1 register		
	Si4432_Write_Register(0x25,0x03);		//write 0x24 to the Clock Recovery Timing Loop Gain 0 register		
	Si4432_Write_Register(0x1D,0x40);		//write 0x40 to the AFC Loop Gearshift Override register		
	Si4432_Write_Register(0x1E,0x0A);		//write 0x0A to the AFC Timing Control register		
	Si4432_Write_Register(0x2A,0x14);		//write 0x20 to the AFC Limiter register		
						
	//ݰĽṹ͵
	//ǰ볤 5bytes 
	Si4432_Write_Register(0x34, 0x0A);	//write 0x0A to the Preamble Length register
    
	//ǰֵ 20bits
	Si4432_Write_Register(0x35,0x2A); 	//write 0x2A to the Preamble Detection Control  register

	//ͷֽ;ÿɱķ鳤ȣЧصĳɶ
	//յݰȵݰ;ֽ֣ͬڳ
	Si4432_Write_Register(0x33,0x02);	//write 0x02 to the Header Control2 register    
	
	//ͬ 0x2DD4
	Si4432_Write_Register(0x36,0x2D);   //write 0x2D to the Sync Word 3 register
	Si4432_Write_Register(0x37,0xD4);	//write 0xD4 to the Sync Word 2 register

	//TXRXݰCRC-16IBMļ
	Si4432_Write_Register(0x30,0x8D);	//write 0x8D to the Data Access Control register
    
	//ýͷ
    Si4432_Write_Register(0x32,0x00 );	//write 0x00 to the Header Control1 register   
             
	//FIFOģʽGFSK
	Si4432_Write_Register(0x71,0x63);	//write 0x63 to the Modulation Mode Control 2 register

	//øGPIOĸRF 
    //Si4432_Write_Register(0x0C,0x12);	//write 0x12 to the GPIO1 Configuration(set the TX state)
	//Si4432_Write_Register(0x0b,0x15);	//write 0x15 to the GPIO2 Configuration(set the RX state) 
	Si4432_Write_Register(0x0C,0x15);	//write 0x15 to the GPIO1 Configuration(set the rX state)
	Si4432_Write_Register(0x0b,0x12);	//write 0x12 to the GPIO0Configuration(set the tX state) 

	//÷ĬSi443xĴ
	//set  cap. bank
	Si4432_Write_Register(0x09,0xD7);	//write 0xD7 to the Crystal Oscillator Load Capacitance register
    
	// Set AGC Override1 Register
	Si4432_Write_Register(0x69,0x60);	//write 0x60 to the AGC Override1 register	

	//TX20dBm
	//Si4432_Write_Register(0x6d,0x1f);	 
	Si4432_Write_Register(0x6d,0x1e);	
	
	//ʹ
	Si4432_Write_Register(0x07,0x05);//write 0x05 to the Operating Function Control 1 register
    
    //Ϊģʽ
    //ʹж 
    // a) ӽӵݰ
    // b) յݰCRC
    Si4432_Write_Register(0x05,0x03); 
    Si4432_Write_Register(0x06,0x00); 
    
    //ȡж״̬regsitersйжnIRQţʹNIRQߵƽ
    Si4432_Read_Register(0x03);	
    Si4432_Read_Register(0x04);	


    //ʹ   
	Si4432_Write_Register(0x07,0x05);//write 0x05 to the Operating Function Control 1 register
		
    Si4432_Write_Register(0x0d,0xf4);	
}



void Read_Reg_Test(void)
{
    unsigned char buf[32]={0};
    unsigned char dat=0;
    
	dat=Si4432_Read_Register(0x75);	
    sprintf(buf,"Si4432_Read_Register(0x75)=%02x\r\n",dat);
    UartSendStr(buf);
    
    
	dat=Si4432_Read_Register(0x76);	
    sprintf(buf,"Si4432_Read_Register(0x76)=%02x\r\n",dat);
    UartSendStr(buf);
    
    														
	dat=Si4432_Read_Register(0x77);	
    sprintf(buf,"Si4432_Read_Register(0x77)=%02x\r\n",dat);
    UartSendStr(buf);
}




void main(void)
{
    char *str="start...\r\n";
    unsigned char buf[128]={0};
    delay_ms(100);
    Io_Init();
    Uart_Init();
    Ext_Inter_Init();
    Si4432_Init();
    #asm("sei")
    UartSendStr(str);
    Read_Reg_Test();
    while (1)
    {
        Key_Scan();
        if(SiRevIntFlag)
        {
            SiRevIntFlag=0;
            Si4432_Get_Data(buf);
        }
        if(SiSendFlag)
        {
            Si4432_Send_Data(RevUartBuf,revCnt);
            revCnt=0;
            SiSendFlag=0;
        }
        //Si4432_Rev_Data(buf);
    };
}
